@@ -33,6 +33,7 @@
 #include "gjs/mem.h"
 
 struct Closure {
+    JSRuntime *runtime;
     JSContext *context;
     GjsMaybeOwned<JSObject *> obj;
 };
@@ -91,6 +92,7 @@ invalidate_js_pointers(GjsClosure *gc)
 
     c->obj.reset();
     c->context = NULL;
+    c->runtime = NULL;
 
     /* Notify any closure reference holders they
      * may want to drop references.
@@ -116,6 +118,25 @@ global_context_finalized(JS::HandleObject obj,
     }
 }
 
+/* Closures have to drop their references to their JS functions in an idle
+ * handler, because otherwise the closure might stop tracing the function object
+ * in the middle of garbage collection. That is not allowed with incremental GC.
+ */
+static gboolean
+closure_clear_idle(void *data)
+{
+    auto closure = static_cast<GjsClosure *>(data);
+    gjs_debug_closure("Clearing closure %p which calls object %p",
+                      &closure->priv, closure->priv.obj.get());
+
+    closure->priv.obj.reset();
+    closure->priv.context = nullptr;
+    closure->priv.runtime = nullptr;
+
+    g_closure_unref(static_cast<GClosure *>(data));
+    return G_SOURCE_REMOVE;
+}
+
 /* Invalidation is like "dispose" - it is guaranteed to happen at
  * finalize, but may happen before finalize. Normally, g_closure_invalidate()
  * is called when the "target" of the closure becomes invalid, so that the
@@ -156,21 +177,17 @@ closure_invalidated(gpointer data,
                       "removing our destroy notifier on global object)",
                       closure);
 
-    c->obj.reset();
-    c->context = nullptr;
+    g_idle_add(closure_clear_idle, closure);
+    g_closure_ref(closure);
 }
 
 static void
 closure_set_invalid(gpointer  data,
                     GClosure *closure)
 {
-    Closure *self = &((GjsClosure*) closure)->priv;
-
-    JS::ExposeObjectToActiveJS(self->obj);
-    self->obj.reset();
-    self->context = nullptr;
-
     GJS_DEC_COUNTER(closure);
+    g_idle_add(closure_clear_idle, closure);
+    g_closure_ref(closure);
 }
 
 static void
@@ -228,6 +245,8 @@ gjs_closure_invoke(GClosure                   *closure,
                           " - closure %p", closure);
     }
 
+    JS_MaybeGC(context);
+
  out:
     JS_EndRequest(context);
 }
@@ -288,6 +307,7 @@ gjs_closure_new(JSContext  *context,
     gc = (GjsClosure*) g_closure_new_simple(sizeof(GjsClosure), NULL);
     c = new (&gc->priv) Closure();
 
+    c->runtime = JS_GetRuntime(context);
     /* The saved context is used for lifetime management, so that the closure will
      * be torn down with the context that created it. The context could be attached to
      * the default context of the runtime using if we wanted the closure to survive
