@@ -34,6 +34,7 @@
 #include "gtype.h"
 #include "param.h"
 #include "gjs/context-private.h"
+#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-private.h"
 #include "gjs/jsapi-wrapper.h"
 #include "gjs/mem.h"
@@ -673,6 +674,22 @@ gjs_fill_method_instance(JSContext       *context,
     return true;
 }
 
+/* Intended for error messages. Return value must be freed */
+static char *
+format_function_name(Function *function,
+                     bool      is_method)
+{
+    auto baseinfo = static_cast<GIBaseInfo *>(function->info);
+    if (is_method)
+        return g_strdup_printf("method %s.%s.%s",
+                               g_base_info_get_namespace(baseinfo),
+                               g_base_info_get_name(g_base_info_get_container(baseinfo)),
+                               g_base_info_get_name(baseinfo));
+    return g_strdup_printf("function %s.%s",
+                           g_base_info_get_namespace(baseinfo),
+                           g_base_info_get_name(baseinfo));
+}
+
 /*
  * This function can be called in 2 different ways. You can either use
  * it to create javascript objects by providing a @js_rval argument or
@@ -680,12 +697,12 @@ gjs_fill_method_instance(JSContext       *context,
  * providing a @r_value argument.
  */
 static bool
-gjs_invoke_c_function(JSContext                              *context,
-                      Function                               *function,
-                      JS::HandleObject                        obj, /* "this" object */
-                      const JS::HandleValueArray&             args,
-                      mozilla::Maybe<JS::MutableHandleValue>& js_rval,
-                      GIArgument                             *r_value)
+gjs_invoke_c_function(JSContext                             *context,
+                      Function                              *function,
+                      JS::HandleObject                       obj, /* "this" object */
+                      const JS::HandleValueArray&            args,
+                      mozilla::Maybe<JS::MutableHandleValue> js_rval,
+                      GIArgument                            *r_value)
 {
     /* These first four are arrays which hold argument pointers.
      * @in_arg_cvalues: C values which are passed on input (in or inout)
@@ -748,30 +765,18 @@ gjs_invoke_c_function(JSContext                              *context,
      * arguments we expect the JS function to take (which does not
      * include PARAM_SKIPPED args).
      *
-     * @js_argc is the number of arguments that were actually passed;
-     * we allow this to be larger than @expected_js_argc for
-     * convenience, and simply ignore the extra arguments. But we
-     * don't allow too few args, since that would break.
+     * @js_argc is the number of arguments that were actually passed.
      */
-
-    if (args.length() < function->expected_js_argc) {
-        auto baseinfo = static_cast<GIBaseInfo *>(function->info);
-        if (is_method) {
-            gjs_throw(context, "Too few arguments to method %s.%s.%s: "
-                      "expected %d, got %" G_GSIZE_FORMAT,
-                      g_base_info_get_namespace(baseinfo),
-                      g_base_info_get_name(g_base_info_get_container(baseinfo)),
-                      g_base_info_get_name(baseinfo),
-                      function->expected_js_argc,
-                      args.length());
-        } else {
-            gjs_throw(context, "Too few arguments to function %s.%s: "
-                      "expected %d, got %" G_GSIZE_FORMAT,
-                      g_base_info_get_namespace(baseinfo),
-                      g_base_info_get_name(baseinfo),
-                      function->expected_js_argc,
-                      args.length());
-        }
+    if (args.length() > function->expected_js_argc) {
+        GjsAutoChar name = format_function_name(function, is_method);
+        JS_ReportWarning(context, "Too many arguments to %s: expected %d, "
+                         "got %" G_GSIZE_FORMAT, name.get(),
+                         function->expected_js_argc, args.length());
+    } else if (args.length() < function->expected_js_argc) {
+        GjsAutoChar name = format_function_name(function, is_method);
+        gjs_throw(context, "Too few arguments to %s: "
+                  "expected %d, got %" G_GSIZE_FORMAT,
+                  name.get(), function->expected_js_argc, args.length());
         return false;
     }
 
@@ -1335,9 +1340,8 @@ function_call(JSContext *context,
     if (priv == NULL)
         return true; /* we are the prototype, or have the wrong class */
 
-    /* COMPAT: mozilla::Maybe gains a much more usable API in future versions */
-    auto m_retval = mozilla::Some<JS::MutableHandleValue>(&retval);
-    success = gjs_invoke_c_function(context, priv, object, js_argv, m_retval,
+    success = gjs_invoke_c_function(context, priv, object, js_argv,
+                                    mozilla::Some<JS::MutableHandleValue>(&retval),
                                     NULL);
     if (success)
         js_argv.rval().set(retval);
@@ -1495,18 +1499,20 @@ struct JSClass gjs_function_class = {
     function_call
 };
 
-JSPropertySpec gjs_function_proto_props[] = {
+static JSPropertySpec gjs_function_proto_props[] = {
     JS_PSG("length", get_num_arguments, JSPROP_PERMANENT),
     JS_PS_END
 };
 
 /* The original Function.prototype.toString complains when
    given a GIRepository function as an argument */
-JSFunctionSpec gjs_function_proto_funcs[] = {
+static JSFunctionSpec gjs_function_proto_funcs[] = {
     JS_FN("toString", function_to_string, 0, 0),
     JS_FS_END
 };
 
+static JSFunctionSpec *gjs_function_static_funcs = nullptr;
+
 static bool
 init_cached_function_data (JSContext      *context,
                            Function       *function,
@@ -1655,66 +1661,33 @@ init_cached_function_data (JSContext      *context,
     return true;
 }
 
+static inline JSObject *
+gjs_builtin_function_get_proto(JSContext *cx)
+{
+    JS::RootedObject global(cx, gjs_get_import_global(cx));
+    return JS_GetFunctionPrototype(cx, global);
+}
+
+GJS_DEFINE_PROTO_FUNCS_WITH_PARENT(function, builtin_function)
+
 static JSObject*
 function_new(JSContext      *context,
              GType           gtype,
              GICallableInfo *info)
 {
     Function *priv;
-    bool found;
 
-    /* put constructor for GIRepositoryFunction() in the global namespace */
-    JS::RootedObject global(context, gjs_get_import_global(context));
-
-    if (!JS_HasProperty(context, global, gjs_function_class.name, &found))
-        return NULL;
-    if (!found) {
-        JSObject *prototype;
-        JS::RootedObject parent_proto(context);
-        JS::RootedValue v_native_function(context);
-
-        JS_GetProperty(context, global, "Function", &v_native_function);
-        JS::RootedObject native_function(context, &v_native_function.toObject());
-        /* We take advantage from that fact that Function.__proto__ is Function.prototype */
-        JS_GetPrototype(context, native_function, &parent_proto);
-
-        prototype = JS_InitClass(context, global,
-                                 /* parent prototype JSObject* for
-                                  * prototype; NULL for
-                                  * Object.prototype
-                                  */
-                                 parent_proto,
-                                 &gjs_function_class,
-                                 /* constructor for instances (NULL for
-                                  * none - just name the prototype like
-                                  * Math - rarely correct)
-                                  */
-                                 gjs_function_constructor,
-                                 /* number of constructor args */
-                                 0,
-                                 /* props of prototype */
-                                 &gjs_function_proto_props[0],
-                                 /* funcs of prototype */
-                                 &gjs_function_proto_funcs[0],
-                                 /* props of constructor, MyConstructor.myprop */
-                                 NULL,
-                                 /* funcs of constructor, MyConstructor.myfunc() */
-                                 NULL);
-        if (prototype == NULL)
-            g_error("Can't init class %s", gjs_function_class.name);
-
-        gjs_debug(GJS_DEBUG_GFUNCTION, "Initialized class %s prototype %p",
-                  gjs_function_class.name, prototype);
-    }
+    JS::RootedObject proto(context);
+    if (!gjs_function_define_proto(context, JS::NullPtr(), &proto))
+        return nullptr;
 
     JS::RootedObject function(context,
-        JS_NewObject(context, &gjs_function_class, global));
+        JS_NewObjectWithGivenProto(context, &gjs_function_class, proto));
     if (function == NULL) {
         gjs_debug(GJS_DEBUG_GFUNCTION, "Failed to construct function");
         return NULL;
     }
 
-
     priv = g_slice_new0(Function);
 
     GJS_INC_COUNTER(function);
@@ -1787,10 +1760,8 @@ gjs_invoke_c_function_uncached(JSContext                  *context,
   if (!init_cached_function_data (context, &function, 0, info))
       return false;
 
-  /* COMPAT: mozilla::Maybe gains a much more usable API in future versions */
-  mozilla::Maybe<JS::MutableHandleValue> m_rval;
-  m_rval.emplace(rval);
-  result = gjs_invoke_c_function(context, &function, obj, args, m_rval, NULL);
+  result = gjs_invoke_c_function(context, &function, obj, args,
+                                 mozilla::Some(rval), NULL);
   uninit_cached_function_data (&function);
   return result;
 }
