@@ -36,6 +36,7 @@
 #include "fundamental.h"
 #include "interface.h"
 #include "gerror.h"
+#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-wrapper.h"
 #include "gjs/jsapi-private.h"
 #include "gjs/mem.h"
@@ -98,6 +99,17 @@ resolve_namespace_object(JSContext       *context,
         return false;
 
     repo = g_irepository_get_default();
+    GList *versions = g_irepository_enumerate_versions(repo, ns_name);
+    unsigned nversions = g_list_length(versions);
+    if (nversions > 1 && !version &&
+        !g_irepository_is_registered(repo, ns_name, NULL)) {
+        GjsAutoChar warn_text = g_strdup_printf("Requiring %s but it has %u "
+                                                "versions available; use "
+                                                "imports.gi.versions to pick one",
+                                                ns_name, nversions);
+        JS_ReportWarning(context, warn_text);
+    }
+    g_list_free_full(versions, g_free);
 
     error = NULL;
     g_irepository_require(repo, ns_name, version, (GIRepositoryLoadFlags) 0, &error);
@@ -227,57 +239,23 @@ struct JSClass gjs_repo_class = {
     repo_finalize
 };
 
-JSPropertySpec gjs_repo_proto_props[] = {
-    JS_PS_END
-};
+static JSPropertySpec *gjs_repo_proto_props = nullptr;
+static JSFunctionSpec *gjs_repo_proto_funcs = nullptr;
+static JSFunctionSpec *gjs_repo_static_funcs = nullptr;
 
-JSFunctionSpec gjs_repo_proto_funcs[] = {
-    JS_FS_END
-};
+GJS_DEFINE_PROTO_FUNCS(repo)
 
 static JSObject*
 repo_new(JSContext *context)
 {
     Repo *priv;
-    bool found;
 
-    JS::RootedObject global(context, gjs_get_import_global(context));
-
-    if (!JS_HasProperty(context, global, gjs_repo_class.name, &found))
-        return NULL;
-    if (!found) {
-        JSObject *prototype;
-        prototype = JS_InitClass(context, global,
-                                 /* parent prototype JSObject* for
-                                  * prototype; NULL for
-                                  * Object.prototype
-                                  */
-                                 JS::NullPtr(),
-                                 &gjs_repo_class,
-                                 /* constructor for instances (NULL for
-                                  * none - just name the prototype like
-                                  * Math - rarely correct)
-                                  */
-                                 gjs_repo_constructor,
-                                 /* number of constructor args */
-                                 0,
-                                 /* props of prototype */
-                                 &gjs_repo_proto_props[0],
-                                 /* funcs of prototype */
-                                 &gjs_repo_proto_funcs[0],
-                                 /* props of constructor, MyConstructor.myprop */
-                                 NULL,
-                                 /* funcs of constructor, MyConstructor.myfunc() */
-                                 NULL);
-        if (prototype == NULL)
-            g_error("Can't init class %s", gjs_repo_class.name);
-
-        gjs_debug(GJS_DEBUG_GREPO, "Initialized class %s prototype %p",
-                  gjs_repo_class.name, prototype);
-    }
+    JS::RootedObject proto(context);
+    if (!gjs_repo_define_proto(context, JS::NullPtr(), &proto))
+        return nullptr;
 
     JS::RootedObject repo(context,
-        JS_NewObject(context, &gjs_repo_class, global));
+        JS_NewObjectWithGivenProto(context, &gjs_repo_class, proto));
     if (repo == NULL) {
         gjs_throw(context, "No memory to create repo object");
         return NULL;
@@ -293,23 +271,29 @@ repo_new(JSContext *context)
     gjs_debug_lifecycle(GJS_DEBUG_GREPO,
                         "repo constructor, obj %p priv %p", repo.get(), priv);
 
-    JS::RootedObject versions(context, JS_NewObject(context, NULL, global));
+    JS::RootedObject versions(context, JS_NewPlainObject(context));
     gjs_object_define_property(context, repo, GJS_STRING_GI_VERSIONS,
                                versions, JSPROP_PERMANENT);
 
-    JS::RootedObject private_ns(context, JS_NewObject(context, NULL, global));
+    /* GLib/GObject/Gio are fixed at 2.0, since we depend on them
+     * internally.
+     */
+    JS::RootedString two_point_oh(context, JS_NewStringCopyZ(context, "2.0"));
+    if (!JS_DefineProperty(context, versions, "GLib", two_point_oh,
+                           JSPROP_PERMANENT))
+        return nullptr;
+    if (!JS_DefineProperty(context, versions, "GObject", two_point_oh,
+                           JSPROP_PERMANENT))
+        return nullptr;
+    if (!JS_DefineProperty(context, versions, "Gio", two_point_oh,
+                           JSPROP_PERMANENT))
+        return nullptr;
+
+    JS::RootedObject private_ns(context, JS_NewPlainObject(context));
     gjs_object_define_property(context, repo,
                                GJS_STRING_PRIVATE_NS_MARKER, private_ns,
                                JSPROP_PERMANENT);
 
-    /* FIXME - hack to make namespaces load, since
-     * gobject-introspection does not yet search a path properly.
-     */
-    {
-        JS::RootedValue value(context);
-        JS_GetProperty(context, repo, "GLib", &value);
-    }
-
     return repo;
 }
 
