@@ -120,7 +120,6 @@ gjs_log(JSContext *context,
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     char *s;
-    JSExceptionState *exc_state;
 
     if (argc != 1) {
         gjs_throw(context, "Must pass a single argument to log()");
@@ -131,11 +130,9 @@ gjs_log(JSContext *context,
 
     /* JS::ToString might throw, in which case we will only log that the value
      * could not be converted to string */
-    exc_state = JS_SaveExceptionState(context);
+    JS::AutoSaveExceptionState exc_state(context);
     JS::RootedString jstr(context, JS::ToString(context, argv[0]));
-    if (jstr != NULL)
-        argv[0].setString(jstr);  // GC root
-    JS_RestoreExceptionState(context, exc_state);
+    exc_state.restore();
 
     if (jstr == NULL) {
         g_message("JS LOG: <cannot convert value to string>");
@@ -162,7 +159,6 @@ gjs_log_error(JSContext *context,
               JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    JSExceptionState *exc_state;
 
     if ((argc != 1 && argc != 2) || !argv[0].isObject()) {
         gjs_throw(context, "Must pass an exception and optionally a message to logError()");
@@ -176,11 +172,9 @@ gjs_log_error(JSContext *context,
     if (argc == 2) {
         /* JS::ToString might throw, in which case we will only log that the
          * value could be converted to string */
-        exc_state = JS_SaveExceptionState(context);
+        JS::AutoSaveExceptionState exc_state(context);
         jstr = JS::ToString(context, argv[1]);
-        if (jstr != NULL)
-            argv[1].setString(jstr);  // GC root
-        JS_RestoreExceptionState(context, exc_state);
+        exc_state.restore();
     }
 
     gjs_log_exception_full(context, argv[0], jstr);
@@ -203,17 +197,11 @@ gjs_print_parse_args(JSContext *context,
 
     str = g_string_new("");
     for (n = 0; n < argv.length(); ++n) {
-        JSExceptionState *exc_state;
-
         /* JS::ToString might throw, in which case we will only log that the
          * value could not be converted to string */
-        exc_state = JS_SaveExceptionState(context);
-
+        JS::AutoSaveExceptionState exc_state(context);
         JS::RootedString jstr(context, JS::ToString(context, argv[n]));
-        if (jstr != NULL)
-            argv[n].setString(jstr); // GC root
-
-        JS_RestoreExceptionState(context, exc_state);
+        exc_state.restore();
 
         if (jstr != NULL) {
             if (!gjs_string_to_utf8(context, JS::StringValue(jstr), &s)) {
@@ -279,6 +267,19 @@ gjs_printerr(JSContext *context,
     return true;
 }
 
+static void
+on_garbage_collect(JSRuntime *rt,
+                   JSGCStatus status,
+                   void      *data)
+{
+    /* We finalize any pending toggle refs before doing any garbage collection,
+     * so that we can collect the JS wrapper objects, and in order to minimize
+     * the chances of objects having a pending toggle up queued when they are
+     * garbage collected. */
+    if (status == JSGC_BEGIN)
+        gjs_object_clear_toggles();
+}
+
 /* Requires request, does not throw error */
 static bool
 gjs_define_promise_object(JSContext       *cx,
@@ -403,11 +404,6 @@ gjs_context_dispose(GObject *object)
         gjs_debug(GJS_DEBUG_CONTEXT,
                   "Destroying JS context");
 
-        /* Finalize any pending toggle refs left over on the main context,
-         * before doing any garbage collection, so that we can collect the JS
-         * wrapper objects */
-        gjs_object_clear_toggles();
-
         JS_BeginRequest(js_context->context);
 
         /* Do a full GC here before tearing down, since once we do
@@ -505,6 +501,8 @@ gjs_context_constructed(GObject *object)
 
     JS_BeginRequest(js_context->context);
 
+    JS_SetGCCallback(js_context->runtime, on_garbage_collect, js_context);
+
     /* set ourselves as the private data */
     JS_SetContextPrivate(js_context->context, js_context);
     JS::RootedObject global(js_context->context);
@@ -810,17 +808,31 @@ gjs_context_eval_file(GjsContext    *js_context,
                       int           *exit_status_p,
                       GError       **error)
 {
-    char *script;
-    size_t script_len;
-    GjsAutoUnref<GFile> file = g_file_new_for_commandline_arg(filename);
+    char     *script = NULL;
+    gsize    script_len;
+    bool ret = true;
 
-    if (!g_file_load_contents(file, nullptr, &script, &script_len, nullptr,
-                              error))
-        return false;
-    GjsAutoChar script_ref = script;
+    GFile *file = g_file_new_for_commandline_arg(filename);
 
-    return gjs_context_eval(js_context, script, script_len, filename,
-                            exit_status_p, error);
+    if (!g_file_query_exists(file, NULL)) {
+        ret = false;
+        goto out;
+    }
+
+    if (!g_file_load_contents(file, NULL, &script, &script_len, NULL, error)) {
+        ret = false;
+        goto out;
+    }
+
+    if (!gjs_context_eval(js_context, script, script_len, filename, exit_status_p, error)) {
+        ret = false;
+        goto out;
+    }
+
+out:
+    g_free(script);
+    g_object_unref(file);
+    return ret;
 }
 
 bool
