@@ -27,6 +27,7 @@
 #include <util/glib.h>
 
 #include "importer.h"
+#include "jsapi-class.h"
 #include "jsapi-wrapper.h"
 #include "mem.h"
 #include "native.h"
@@ -53,11 +54,13 @@ typedef struct {
     unsigned int index;
 } ImporterIterator;
 
-extern const js::Class gjs_importer_class;
+extern const js::Class gjs_importer_real_class;
 
 /* Bizarrely, the API for safely casting const js::Class * to const JSClass *
  * is called "js::Jsvalify" */
-GJS_DEFINE_PRIV_FROM_JS(Importer, (*js::Jsvalify(&gjs_importer_class)))
+static const JSClass gjs_importer_class = *js::Jsvalify(&gjs_importer_real_class);
+
+GJS_DEFINE_PRIV_FROM_JS(Importer, gjs_importer_class)
 
 static bool
 importer_to_string(JSContext *cx,
@@ -101,7 +104,7 @@ define_meta_properties(JSContext       *context,
      * on the root importer
      */
     parent_is_module = parent && JS_InstanceOf(context, parent,
-        js::Jsvalify(&gjs_importer_class), NULL);
+                                               &gjs_importer_class, nullptr);
 
     gjs_debug(GJS_DEBUG_IMPORTER, "Defining parent %p of %p '%s' is mod %d",
               parent.get(), module_obj.get(),
@@ -385,7 +388,7 @@ load_module_init(JSContext       *context,
         }
     }
 
-    JS::RootedObject module_obj(context, JS_NewObject(context, NULL));
+    JS::RootedObject module_obj(context, JS_NewPlainObject(context));
     GjsAutoUnref<GFile> file = g_file_new_for_commandline_arg(full_path);
     if (!import_file (context, "__init__", file, module_obj))
         return module_obj;
@@ -462,7 +465,7 @@ import_file_on_module(JSContext       *context,
     bool retval = false;
     char *full_path = NULL;
 
-    JS::RootedObject module_obj(context, JS_NewObject(context, NULL));
+    JS::RootedObject module_obj(context, JS_NewPlainObject(context));
 
     if (!define_import(context, obj, module_obj, name))
         goto out;
@@ -855,7 +858,7 @@ importer_finalize(js::FreeOp *fop,
  * instances of the object, and to the prototype that instances of the
  * class have.
  */
-const js::Class gjs_importer_class = {
+const js::Class gjs_importer_real_class = {
     "GjsFileImporter",
     JSCLASS_HAS_PRIVATE |
     JSCLASS_IMPLEMENTS_BARRIERS,
@@ -889,62 +892,30 @@ const js::Class gjs_importer_class = {
     }
 };
 
-JSPropertySpec gjs_importer_proto_props[] = {
-    JS_PS_END
-};
+static JSPropertySpec *gjs_importer_proto_props = nullptr;
+static JSFunctionSpec *gjs_importer_static_funcs = nullptr;
 
 JSFunctionSpec gjs_importer_proto_funcs[] = {
     JS_FS("toString", importer_to_string, 0, 0),
     JS_FS_END
 };
 
+GJS_DEFINE_PROTO_FUNCS(importer)
+
 static JSObject*
 importer_new(JSContext *context,
              bool       is_root)
 {
     Importer *priv;
-    bool found;
 
-    JS::RootedObject global(context, gjs_get_import_global(context));
-
-    if (!JS_HasProperty(context, global, gjs_importer_class.name, &found))
-        g_error("HasProperty call failed creating importer class");
-
-    if (!found) {
-        JSObject *prototype;
-        prototype = JS_InitClass(context, global,
-                                 /* parent prototype JSObject* for
-                                  * prototype; NULL for
-                                  * Object.prototype
-                                  */
-                                 JS::NullPtr(),
-                                 js::Jsvalify(&gjs_importer_class),
-                                 /* constructor for instances (NULL for
-                                  * none - just name the prototype like
-                                  * Math - rarely correct)
-                                  */
-                                 gjs_importer_constructor,
-                                 /* number of constructor args */
-                                 0,
-                                 /* props of prototype */
-                                 &gjs_importer_proto_props[0],
-                                 /* funcs of prototype */
-                                 &gjs_importer_proto_funcs[0],
-                                 /* props of constructor, MyConstructor.myprop */
-                                 NULL,
-                                 /* funcs of constructor, MyConstructor.myfunc() */
-                                 NULL);
-        if (prototype == NULL)
-            g_error("Can't init class %s", gjs_importer_class.name);
-
-        gjs_debug(GJS_DEBUG_IMPORTER, "Initialized class %s prototype %p",
-                  gjs_importer_class.name, prototype);
-    }
+    JS::RootedObject proto(context);
+    if (!gjs_importer_define_proto(context, JS::NullPtr(), &proto))
+        g_error("Error creating importer prototype");
 
     JS::RootedObject importer(context,
-        JS_NewObject(context, js::Jsvalify(&gjs_importer_class), global));
+        JS_NewObjectWithGivenProto(context, &gjs_importer_class, proto));
     if (importer == NULL)
-        g_error("No memory to create importer importer");
+        g_error("No memory to create importer");
 
     priv = g_slice_new0(Importer);
     priv->is_root = is_root;
