@@ -30,13 +30,6 @@
 #include "jsapi-util.h"
 #include "jsapi-wrapper.h"
 
-#ifdef __GNUC__
-#define GNUC_ALWAYS_INLINE __attribute__((always_inline))
-#else
-#define GNUC_ALWAYS_INLINE
-#endif
-
-GNUC_ALWAYS_INLINE
 static inline bool
 check_nullable(const char*& fchar,
                const char*& fmt_string)
@@ -53,7 +46,6 @@ check_nullable(const char*& fchar,
 
 /* This preserves the previous behaviour of gjs_parse_args(), but maybe we want
  * to use JS::ToBoolean instead? */
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -72,7 +64,6 @@ assign(JSContext      *cx,
 
 /* This preserves the previous behaviour of gjs_parse_args(), but maybe we want
  * to box primitive types instead of throwing? */
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext              *cx,
        char                    c,
@@ -91,7 +82,6 @@ assign(JSContext              *cx,
     ref.set(&value.toObject());
 }
 
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -113,7 +103,7 @@ assign(JSContext      *cx,
         throw g_strdup_printf("Wrong type for %c, got char**", c);
     }
 }
-GNUC_ALWAYS_INLINE
+
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -129,7 +119,6 @@ assign(JSContext      *cx,
         throw g_strdup("Couldn't convert to integer");
 }
 
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -150,7 +139,6 @@ assign(JSContext      *cx,
     *ref = num;
 }
 
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -166,7 +154,6 @@ assign(JSContext      *cx,
         throw g_strdup("Couldn't convert to 64-bit integer");
 }
 
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -186,7 +173,6 @@ assign(JSContext      *cx,
  * prevent instantiation for any other types besides pointer-to-enum */
 template<typename T,
          typename std::enable_if<std::is_enum<T>::value, int>::type = 0>
-GNUC_ALWAYS_INLINE
 static inline void
 assign(JSContext      *cx,
        char            c,
@@ -212,7 +198,6 @@ template<typename T,
 static inline void
 free_if_necessary(T param_ref) {}
 
-GNUC_ALWAYS_INLINE
 static inline void
 free_if_necessary(JS::MutableHandleObject param_ref)
 {
@@ -222,7 +207,6 @@ free_if_necessary(JS::MutableHandleObject param_ref)
     param_ref.set(NULL);
 }
 
-GNUC_ALWAYS_INLINE
 static inline void
 free_if_necessary(char **param_ref)
 {
@@ -382,7 +366,7 @@ gjs_parse_call_args(JSContext    *cx,
 {
     const char *fmt_iter, *fmt_required, *fmt_optional;
     unsigned n_required = 0, n_total = 0;
-    bool ignore_trailing_args = false, retval;
+    bool optional_args = false, ignore_trailing_args = false, retval;
     char **parts;
 
     if (*format == '!') {
@@ -394,6 +378,7 @@ gjs_parse_call_args(JSContext    *cx,
         switch (*fmt_iter) {
         case '|':
             n_required = n_total;
+            optional_args = true;
             continue;
         case '?':
             continue;
@@ -402,7 +387,7 @@ gjs_parse_call_args(JSContext    *cx,
         }
     }
 
-    if (n_required == 0)
+    if (!optional_args)
         n_required = n_total;
 
     g_assert(((void) "Wrong number of parameters passed to gjs_parse_call_args()",
