@@ -22,13 +22,17 @@
 
 #include <config.h>
 
+#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include "cairo-private.h"
 
-GJS_DEFINE_PROTO("CairoImageSurface", cairo_image_surface, JSCLASS_BACKGROUND_FINALIZE)
+static JSObject *gjs_cairo_image_surface_get_proto(JSContext *);
+
+GJS_DEFINE_PROTO_WITH_PARENT("ImageSurface", cairo_image_surface,
+                             cairo_surface, JSCLASS_BACKGROUND_FINALIZE)
 
 GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_image_surface)
 {
@@ -87,8 +91,10 @@ createFromPNG_func(JSContext *context,
     if (!gjs_cairo_check_status(context, cairo_surface_status(surface), "surface"))
         return false;
 
+    JS::RootedObject proto(context, gjs_cairo_image_surface_get_proto(context));
     JS::RootedObject surface_wrapper(context,
-        JS_NewObject(context, &gjs_cairo_image_surface_class));
+        JS_NewObjectWithGivenProto(context, &gjs_cairo_image_surface_class,
+                                   proto));
     if (!surface_wrapper) {
         gjs_throw(context, "failed to create surface");
         return false;
@@ -206,6 +212,11 @@ JSFunctionSpec gjs_cairo_image_surface_proto_funcs[] = {
     JS_FS_END
 };
 
+JSFunctionSpec gjs_cairo_image_surface_static_funcs[] = {
+    JS_FS("createFromPNG", createFromPNG_func, 1, GJS_MODULE_PROP_FLAGS),
+    JS_FS_END
+};
+
 JSObject *
 gjs_cairo_image_surface_from_surface(JSContext       *context,
                                      cairo_surface_t *surface)
@@ -214,8 +225,10 @@ gjs_cairo_image_surface_from_surface(JSContext       *context,
     g_return_val_if_fail(surface != NULL, NULL);
     g_return_val_if_fail(cairo_surface_get_type(surface) == CAIRO_SURFACE_TYPE_IMAGE, NULL);
 
+    JS::RootedObject proto(context, gjs_cairo_image_surface_get_proto(context));
     JS::RootedObject object(context,
-        JS_NewObject(context, &gjs_cairo_image_surface_class));
+        JS_NewObjectWithGivenProto(context, &gjs_cairo_image_surface_class,
+                                   proto));
     if (!object) {
         gjs_throw(context, "failed to create image surface");
         return NULL;
@@ -226,12 +239,3 @@ gjs_cairo_image_surface_from_surface(JSContext       *context,
     return object;
 }
 
-void
-gjs_cairo_image_surface_init(JSContext       *cx,
-                             JS::HandleObject proto)
-{
-
-    if (!JS_DefineFunction(cx, proto, "createFromPNG", createFromPNG_func,
-                           1, GJS_MODULE_PROP_FLAGS))
-        return;
-}
