@@ -22,6 +22,7 @@
 
 #include <config.h>
 
+#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util-args.h"
 #include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
@@ -30,7 +31,10 @@
 #if CAIRO_HAS_PDF_SURFACE
 #include <cairo-pdf.h>
 
-GJS_DEFINE_PROTO("CairoPDFSurface", cairo_pdf_surface, JSCLASS_BACKGROUND_FINALIZE)
+static JSObject *gjs_cairo_pdf_surface_get_proto(JSContext *);
+
+GJS_DEFINE_PROTO_WITH_PARENT("PDFSurface", cairo_pdf_surface,
+                             cairo_surface, JSCLASS_BACKGROUND_FINALIZE)
 
 GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_pdf_surface)
 {
@@ -79,6 +83,8 @@ JSFunctionSpec gjs_cairo_pdf_surface_proto_funcs[] = {
     JS_FS_END
 };
 
+JSFunctionSpec gjs_cairo_pdf_surface_static_funcs[] = { JS_FS_END };
+
 JSObject *
 gjs_cairo_pdf_surface_from_surface(JSContext       *context,
                                    cairo_surface_t *surface)
@@ -87,8 +93,10 @@ gjs_cairo_pdf_surface_from_surface(JSContext       *context,
     g_return_val_if_fail(surface != NULL, NULL);
     g_return_val_if_fail(cairo_surface_get_type(surface) == CAIRO_SURFACE_TYPE_PDF, NULL);
 
+    JS::RootedObject proto(context, gjs_cairo_pdf_surface_get_proto(context));
     JS::RootedObject object(context,
-        JS_NewObject(context, &gjs_cairo_pdf_surface_class));
+        JS_NewObjectWithGivenProto(context, &gjs_cairo_pdf_surface_class,
+                                   proto));
     if (!object) {
         gjs_throw(context, "failed to create pdf surface");
         return NULL;
